package org.one.stone.soup.dfml;

import java.io.IOException;
import java.util.Vector;

import org.one.stone.soup.xml.XmlElement;

public class DfmlCompiler {

	private int NOT_FOUND = -1;

	private DfmlDataDefinition dfmlData;

	private int tagCount;
	private int callFnCount;
	private int clauseFnCount;
	private int dataCount;
	
	private boolean generateInterface;
	private boolean generateTemplate;
	private boolean generateStub;
	
public DfmlCompiler(boolean generateInterface,boolean generateTemplate,boolean generateStub)
{
	this.generateInterface = generateInterface;
	this.generateTemplate = generateTemplate;
	this.generateStub = generateStub;	
}
	
public DfmlDataDefinition compile(XmlElement doc ,String newPackageName,String newClassName) {

	dfmlData = new DfmlDataDefinition(DfmlDataDefinition.STRING_FLAVOUR);
	
	dfmlData.packageName = newPackageName;
	dfmlData.className = newClassName;

	createData(doc);

//	System.out.println("Pre compile done");
	
	DfmlGenerator gen = new DfmlGenerator(dfmlData);
	try{
		gen.generate( generateInterface,generateTemplate,generateStub );
	}catch(IOException ie){}// Dump exception as only occurs in FILE_FLAVOUR

	return dfmlData;
}
public DfmlDataDefinition compile(XmlElement doc ,String newPackageName,String newClassName, String targetPath,String sourceFile) throws IOException{

	dfmlData = new DfmlDataDefinition();
	
	dfmlData.sourceFile = sourceFile;
	dfmlData.targetPath = targetPath;
	dfmlData.packageName = newPackageName;
	dfmlData.className = newClassName;

	createData(doc);

//	System.out.println("Pre compile done");
	
	DfmlGenerator gen = new DfmlGenerator(dfmlData);
	gen.generate(generateInterface,generateTemplate,generateStub);

	return dfmlData;
}
private XmlElement createData(XmlElement element) {

	System.out.println("element :"+element.getName()+" lines:"+element.getStartLineNo()+" to "+element.getEndLineNo() );

	String name = element.getName().toUpperCase();
	XmlElement currentTag = new XmlElement(name);
	XmlElement newTag = null;
	
	dfmlData.tags.addElement(currentTag);
	int startTagCount = tagCount;
	int elsePos=0;
	XmlElement elseTag=null;

	int found;
	
//----------------------------------------------------------------------------
				if(name.equals(DfmlTags.IF))
				{
//	IF
//		CLAUSE TRUE
					found=findClauseFn(element.getAttributeValueByName(DfmlTags.IF_TRUE));
		
					if(found==NOT_FOUND)
					{	
						currentTag.addAttribute( DfmlTags._CLAUSE,""+clauseFnCount );
						dfmlData.clauseFns.addElement( element.getAttributeValueByName(DfmlTags.IF_TRUE) );
						clauseFnCount++;

						//writeIfStub(element.getAttributes());
					}
					else
					{
						currentTag.addAttribute( DfmlTags._CLAUSE,""+found );
					}
					
					tagCount++;
					tagCount++;// tag added later
				}
				else if(name.equals(DfmlTags.FOR))
				{
//	FOR
//		CALL INIT
					found=findCallFn(element.getAttributeValueByName(DfmlTags.FOR_INIT));
		
					if(found==NOT_FOUND)
					{	
						currentTag.addAttribute( DfmlTags._CALL,""+callFnCount );
						dfmlData.callFns.addElement( element.getAttributeValueByName(DfmlTags.FOR_INIT) );
						callFnCount++;

						//writeForStub(element.getAttributes());
					}
					else
					{
						currentTag.addAttribute( DfmlTags._CALL,""+found );
					}
					
					tagCount++;
//		CLAUSE WHILE
					startTagCount = tagCount;

					found=findClauseFn(element.getAttributeValueByName(DfmlTags.FOR_WHILE));
		
					if(found==NOT_FOUND)
					{	
						currentTag.addAttribute( DfmlTags._CLAUSE,""+clauseFnCount );
						dfmlData.clauseFns.addElement( element.getAttributeValueByName(DfmlTags.FOR_WHILE) );
						clauseFnCount++;
					}
					else
					{
						currentTag.addAttribute( DfmlTags._CLAUSE,""+found );
					}
					
					tagCount++;
					tagCount++;// tag added later
				}
				else if(name.equals(DfmlTags.INSERT))
				{
//	INSERT
//		CALL FN
					found=findCallFn(element.getAttributeValueByName(DfmlTags.INSERT_FROM));
		
					if(found==NOT_FOUND)
					{	
						currentTag.addAttribute( DfmlTags._CALL,""+callFnCount );
						dfmlData.callFns.addElement( element.getAttributeValueByName(DfmlTags.INSERT_FROM) );
						callFnCount++;

						//writeInsertStub(element.getAttributes());
					}
					else
					{
						currentTag.addAttribute( DfmlTags._CALL,""+found );
					}
					
					tagCount++;
				}
				else if(name.equals(DfmlTags.ELSE))
				{
					tagCount++; //tag added later
				}			
//----------------------------------------------------------------------------
	
	if(element.getElements()!=null)
	{
		Vector elements = element.getElements();
		
		for(int loop=0;loop<elements.size();loop++)
		{
			if(elements.elementAt(loop) instanceof  String)
			{
				newTag = new XmlElement(DfmlTags._DATA,""+elements.elementAt(loop));
				dfmlData.tags.addElement( newTag );
				
				found=findData((String)elements.elementAt(loop));
		
				if(found==NOT_FOUND)
				{	
					newTag.addAttribute( DfmlTags._DATA,""+dataCount );
					dfmlData.data.addElement( elements.elementAt(loop) );
					dataCount++;
				}
				else
				{
					newTag.addAttribute( DfmlTags._DATA,""+found );
				}

				tagCount++;

			}
			else
			{
				if( ((XmlElement)elements.elementAt(loop)).getName().toUpperCase().equals(DfmlTags.ELSE) )
				{
					elseTag = createData((XmlElement)elements.elementAt(loop));// recurse
					elsePos=tagCount;
				}
				else
				{
					createData((XmlElement)elements.elementAt(loop));// recurse
				}
			}
		}
	}

//----------------------------------------------------------------------------
				if(name.equals(DfmlTags.IF))
				{
//		GOTO FALSE *
					if(elsePos>0)
					{
						currentTag.addAttribute( DfmlTags._GOTO,""+elsePos );
						elseTag.addAttribute(DfmlTags._GOTO,""+tagCount);
					}
					else
						currentTag.addAttribute( DfmlTags._GOTO,""+tagCount );
					//tagCount++;
				}
				else if(name.equals(DfmlTags.FOR))
				{
//		...
//		CALL NEXT
					newTag = new XmlElement( DfmlTags.FOR_NEXT );
					dfmlData.tags.addElement(newTag);

					found=findCallFn(element.getAttributeValueByName(DfmlTags.FOR_NEXT));
		
					if(found==NOT_FOUND)
					{	
						newTag.addAttribute( DfmlTags._CALL,""+callFnCount );
						dfmlData.callFns.addElement( element.getAttributeValueByName(DfmlTags.FOR_NEXT) );
						callFnCount++;
					}
					else
					{
						newTag.addAttribute( DfmlTags._CALL,""+found );
					}
					
					tagCount++;
//		GOTO *
					newTag.addAttribute( DfmlTags._GOTO,""+startTagCount );
					tagCount++;
//	*	GOTO FALSE
					currentTag.addAttribute( DfmlTags._GOTO,""+tagCount );
					//tagCount++;
				}
//----------------------------------------------------------------------------

//	System.out.println("element :"+element.getName()+" done." );

	return currentTag;

}
private int findCallFn(String data) {

	for(int loop=0;loop<dfmlData.callFns.size();loop++)
	{
		if( dfmlData.callFns.elementAt(loop) instanceof String )
		{
			if( ((String)dfmlData.callFns.elementAt(loop)).equals(data) )
				return loop;
		}
	}
	
	return NOT_FOUND;
}
private int findClauseFn(String data) {

	for(int loop=0;loop<dfmlData.clauseFns.size();loop++)
	{
		if( ((String)dfmlData.clauseFns.elementAt(loop)).equals(data) )
			return loop;
	}
	
	return NOT_FOUND;
}
private int findData(String string) {

	for(int loop=0;loop<dfmlData.data.size();loop++)
	{
		if( ((String)dfmlData.data.elementAt(loop)).equals(string) )
			return loop;
	}
	
	return NOT_FOUND;
}
}
