/*
 * Created on 29-Oct-2004
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package org.one.stone.soup.io;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.one.stone.soup.constants.TimeConstants;
import org.one.stone.soup.util.TimeWatch;

/*
 * Wet-Wired.com Library Version Date 29-Oct-2004
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.onestonesoup.org/OSSLicense.html
 */
/**
 * A light weight wrapper for an InputStream and OutputStream
 * that comprise a connection to a service.
 *
 * @author nikcross
 *
 */
public class StreamConnection implements Connection,Runnable,ActionListener{
	private InputStream inputStream;
	private OutputStream outputStream;
	private long lastWrite=0;
	private long timeoutTime = -1;
	private long dataSize=0;
	private ConnectionListener listener;
	private String alias;
	private TimeWatch timeoutTimer;
	/**
	 *
	 */
	public StreamConnection(InputStream from,OutputStream to,String alias,long timeoutTimeMilliSeconds) {
		super();
		
		this.timeoutTime = timeoutTimeMilliSeconds;
		this.inputStream = from;
		this.outputStream = to;
	}
	
	public void connect(ConnectionListener listener,String alias)
	{
		this.listener = listener;
		this.alias = alias;
		
		new Thread(this,alias).start();
		
		if(timeoutTime>0)
		{
			timeoutTimer = new TimeWatch(timeoutTime);
			timeoutTimer.addActionListener(this);
			timeoutTimer.start();
		}
	}
	
	public void disconnect()
	{
		running = false;
		if(timeoutTimer!=null)
		{
			timeoutTimer.stop();
			timeoutTimer = null;
		}
		try{
			inputStream.close();
		}catch(Exception e){}
		try{
			outputStream.close();
		}catch(Exception e){}
	}
	
	public long getDataSentSize()
	{
		return dataSize;
	}
	
	public String getAlias()
	{
		return alias;
	}
	
	public void close() throws IOException
	{
		inputStream.close();
		outputStream.close();
		running = false;
	}
	
	boolean running = false;

	public void run()
	{
		try{
			running = true;
			
			lastWrite = System.currentTimeMillis();

			byte data[] = new byte[1000];

			int count = 0;
			int size = inputStream.read(data);

			while(running && size != -1)
			{
				if(size>0)
				{
					//System.out.println(getAlias()+" sent "+new String(data,0,size));
					outputStream.write( data,0,size );
					outputStream.flush();
					dataSize+=size;
					size=0;
					
					lastWrite = System.currentTimeMillis();
					if(timeoutTime>0)
					{
						timeoutTimer.reset();
					}
				}

				count=0;
				while(count<10 && inputStream.available()==0)
				{
					try{Thread.sleep(100);}catch(Exception e){}
					count++;
				}
				size = inputStream.available();
				if(size<=0)
				{
					continue;
				}
				else if(size>data.length)
				{
					size = data.length;
				}
				size = inputStream.read(data,0,size);
			}
			
			running = false;
		}
		catch(Exception e)
		{
			//e.printStackTrace();
		}
		finally
		{
			listener.connectionClosed( this );
		}
	}

	public long getLastWriteTime() {
		return lastWrite;
	}

	public InputStream getInputStream() {
		return inputStream;
	}

	public OutputStream getOutputStream() {
		return outputStream;
	}

	public void actionPerformed(ActionEvent e) {
		disconnect();
	}
}
