package org.one.stone.soup.screen.recorder;

import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.zip.GZIPOutputStream;

import org.one.stone.soup.util.Queue;

public abstract class ScreenRecorder implements Runnable{

	private Rectangle recordArea;
	
	private int rawDataSize;
	private int[] rawData;
	private int[] newRawData;
	
	private OutputStream oStream;
	
	private boolean recording = false;
	private boolean running = false;
	private boolean sendKeyFrame=true;

	private long startTime;
	private long frameTime;
	
	private ScreenRecorderListener listener;
	
	private class DataPack
	{
		public DataPack(int[] oldData,int[] newData)
		{
			this.oldData = oldData;
			this.newData = newData;
		}
		
		public int[] oldData;
		public int[] newData;
	}
	
	private class StreamPacker implements Runnable
	{	
		private ScreenRecorder recorder;
		private Queue queue = new Queue();
		public StreamPacker( ScreenRecorder recorder )
		{
			this.recorder = recorder;
			new Thread(this,"Stream Packer").start();
		}
		
		public void packToStream( DataPack pack )
		{
			queue.post( pack );
		}
		
		public void run()
		{
			while(recording)
			{
				while( queue.isEmpty()==false )
				{
					DataPack pack = (DataPack)queue.get();
					try{
						recorder.packToStream(pack.oldData,pack.newData);
					}
					catch(Exception e)
					{
						e.printStackTrace();
						try{ oStream.close(); }catch(Exception e2){}
						break;
					}
				}
				int count=0;
				while(queue.isEmpty()==true && count<100)
				{
					try{Thread.sleep(50);}catch(Exception e){}
					count++;
				}
				if(count==100)
				{
					recording=false;
					break;
				}
			}
		}
	}
	private StreamPacker streamPacker;
	
	public ScreenRecorder(OutputStream oStream,ScreenRecorderListener listener)
	{
		
		this.listener = listener;
		this.oStream = oStream;		
	}

	public void triggerRecordingStop()
	{
		recording = false;
	}
	
	public synchronized void run()
	{
		startTime = System.currentTimeMillis();
		
		recording = true;
		running = true;
		
		streamPacker = new StreamPacker(this);
		
		while( recording )
		{
			try{ Thread.sleep(50); }
			catch(Exception e){	}
			
			try{
				recordFrame();
			}
			catch(Exception e)
			{
				e.printStackTrace();
				try{ oStream.close(); }catch(Exception e2){}
				break;
			}
		}
		
		running = false;
		recording = false;
		
		listener.recordingStopped();
	}

	public abstract Rectangle initialiseScreenCapture();	
	public abstract BufferedImage captureScreen(Rectangle recordArea);
	
	public void sendKeyFrame()
	{
		sendKeyFrame=true;
	}
	
	public void recordFrame() throws IOException
	{
		//long t1 = System.currentTimeMillis();
		BufferedImage bImage = captureScreen(recordArea);
		frameTime = System.currentTimeMillis()-startTime;
		//long t2 = System.currentTimeMillis();

		if(rawData==null || sendKeyFrame==true)
		{
			rawDataSize = recordArea.width*recordArea.height;
			rawData = new int[rawDataSize];
		}

		newRawData = new int[rawDataSize];

		bImage.getRGB(0,0,recordArea.width,recordArea.height,newRawData,0,recordArea.width);
		//long t3 = System.currentTimeMillis();
		
		//packToStream(rawData,newRawData);
		streamPacker.packToStream( new DataPack(rawData,newRawData) );
		//long t4 = System.currentTimeMillis();
		
		/*System.out.println("Times");
		System.out.println("  capture time:"+(t2-t1));
		System.out.println("  data grab time:"+(t3-t2));
		System.out.println("  pack time:"+(t4-t3));*/
		
		rawData = newRawData;
		
		listener.frameRecorded(sendKeyFrame);
		sendKeyFrame = false;
	}
	
	private void packToStream( int oldData[],int[] newData ) throws IOException
	{
		byte[] packed = new byte[newData.length*4];
		
		int inCursor = 0;
		int outCursor = 0;
		int blocks = 0;

		boolean inBlock = true;
		int blockSize = 0;
		byte blockRed = 0;
		byte blockGreen = 0;
		byte blockBlue = 0;
		
		int blankBlocks = 0;
		
		int uncompressedCursor = -1;
		
		byte red;
		byte green;
		byte blue;
		
		boolean hasChanges = false;
		boolean lastEntry = false;
		
		while(inCursor<newData.length)
		{
			if(inCursor==newData.length-1)
			{
				lastEntry = true;
			}
			
			if(newData[inCursor]==oldData[inCursor])
			{
				red=0;
				green=0;
				blue=0;
			}
			else
			{
				red = (byte) ((newData[inCursor] & 0x00FF0000) >>> 16);
				green = (byte) ((newData[inCursor] & 0x0000FF00) >>> 8);
				blue = (byte) ((newData[inCursor] & 0x000000FF));
				
				if(red==0 && green==0 && blue==0)
				{
					blue = 1;
				}
			}
			
			if(
				blockRed == red &&
				blockGreen == green &&
				blockBlue == blue
			)
			{
				if(inBlock==false)
				{
					if(uncompressedCursor>-1)
					{
//						System.out.println("Uncomp:"+blockSize);
						blocks++;
						hasChanges=true;
						packed[uncompressedCursor] = (byte)(blockSize + 0x80);
					}
					inBlock=true;
					blockSize = 0;
					blankBlocks = 0;
				}
				else if(blockSize==126 || lastEntry==true)
				{
					if(
						blockRed==0 &&
						blockGreen==0 &&
						blockBlue==0
					)
					{
//						System.out.println("Blank:"+126);

						if(blankBlocks>0)
						{
							blankBlocks++;
							packed[outCursor-1] = (byte)blankBlocks;
						}
						else
						{
							blocks++;
							blankBlocks++;
							packed[outCursor] = (byte)0xFF;
							outCursor++;
							packed[outCursor] = (byte)blankBlocks;
							outCursor++;
						}
						if(blankBlocks==255)
						{
							blankBlocks = 0;
						}
					}
					else
					{
//						System.out.println("Comp:"+blockSize);
						blocks++;
						hasChanges=true;
						packed[outCursor] = (byte)blockSize;
						outCursor++;
						packed[outCursor] = blockRed;
						outCursor++;
						packed[outCursor] = blockGreen;
						outCursor++;
						packed[outCursor] = blockBlue;
						outCursor++;
						
						blankBlocks = 0;
					}			
					inBlock=true;
					blockSize = 0;
				}
				
				/*blockRed = red;
				blockGreen = green;
				blockBlue = blue;*/
			}
			else
			{
				if(inBlock==true)
				{
//					System.out.println("Comp:"+blockSize);
					if(blockSize>0)
					{
						blocks++;
						hasChanges=true;
						packed[outCursor] = (byte)blockSize;
						outCursor++;
						packed[outCursor] = blockRed;
						outCursor++;
						packed[outCursor] = blockGreen;
						outCursor++;
						packed[outCursor] = blockBlue;
						outCursor++;
					}

					uncompressedCursor = -1;
					inBlock=false;
					blockSize = 0;
					
					blankBlocks = 0;
				}
				else if(blockSize==126 || lastEntry==true)
				{
					if(uncompressedCursor>-1)
					{
//						System.out.println("Uncomp:"+blockSize);
						blocks++;
						hasChanges=true;
						packed[uncompressedCursor] = (byte)(blockSize + 0x80);
					}
					
					uncompressedCursor = -1;
					inBlock=false;
					blockSize=0;
					
					blankBlocks = 0;					
				}


				if(uncompressedCursor == -1)
				{
					uncompressedCursor = outCursor;
					outCursor++;
				}
				
				packed[outCursor] = red;
				outCursor++;
				packed[outCursor] = green;
				outCursor++;
				packed[outCursor] = blue;
				outCursor++;
				
				blockRed = red;
				blockGreen = green;
				blockBlue = blue;
			}
			
/*			blockRed = red;
			blockGreen = green;
			blockBlue = blue;*/
			
			inCursor++;
			blockSize++;
			
/*			if(blocks>10)
			{
				break;
			}
*/
		}

//		try{
			oStream.write( ((int)frameTime & 0xFF000000) >>>24 );
			oStream.write( ((int)frameTime & 0x00FF0000) >>>16 );
			oStream.write( ((int)frameTime & 0x0000FF00) >>>8 );
			oStream.write( ((int)frameTime & 0x000000FF) );			
			//System.out.println("ft:"+frameTime);
			
			if(hasChanges==false)
			{
				//System.out.println((inCursor*4)+" Packed Code:"+0);
				oStream.write(0);
				oStream.flush();
				
				return;
			}
			else
			{
				//System.out.println((inCursor*4)+" Packed Code:"+1);
				oStream.write(1);
				oStream.flush();
			}
/*		}
		catch(Exception e)
		{
			e.printStackTrace();
		}*/
				
		//System.out.println((inCursor*4)+" Packed To: "+outCursor);
		
		ByteArrayOutputStream bO = new ByteArrayOutputStream();
		/*ZipOutputStream zO = new ZipOutputStream(bO);
		ZipEntry zE = new ZipEntry("data");*/
				
		byte[] bA = new byte[0];
		
//		try{
			GZIPOutputStream zO = new GZIPOutputStream(bO);
			//zO.putNextEntry(zE);

			zO.write(packed,0,outCursor);
			zO.close();
			bO.close();
			
			bA = bO.toByteArray();
			
			oStream.write( (bA.length & 0xFF000000) >>>24 );
			oStream.write( (bA.length & 0x00FF0000) >>>16 );
			oStream.write( (bA.length & 0x0000FF00) >>>8 );
			oStream.write( (bA.length & 0x000000FF) );
			
			oStream.write(bA);
			oStream.flush();
/*		}
		catch(Exception e)
		{
			e.printStackTrace();
		}*/
		
		//System.out.println((inCursor*4)+" Zipped To: "+bA.length);
		return;
	}

	public void startRecording()
	{		
		recordArea = initialiseScreenCapture();
		
		try{

			oStream.write( (recordArea.width & 0x0000FF00) >>>8 );
			oStream.write( (recordArea.width & 0x000000FF) );
		
			oStream.write( (recordArea.height & 0x0000FF00) >>>8 );
			oStream.write( (recordArea.height & 0x000000FF) );
		}
		catch(Exception e)
		{
			e.printStackTrace();		
		}		
		
		new Thread(this,"Screen Recorder").start();
	}

	public void stopRecording()
	{
		triggerRecordingStop();
		
		/*while(running == true)
		{
			try{ Thread.sleep(100);
			}catch(Exception e){}
		}*/
		
		try{
			oStream.flush();
			oStream.close();
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}

	public Rectangle getRecordArea() {
		return recordArea;
	}
	
	public boolean isRecording()
	{
		return recording;
	}
}
