package org.one.stone.soup.util;

/*
 * Wet-Wired.com Library Version 2.1
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.onestonesoup.org/OSSLicense.html
 */

/**
 * @author nikcross
 *
 */

// TODO
public abstract class Recycler {

	private int startSize;				// Start size of the resource pool
	private int maxSize;				// Maximum size of the resource pool
	private int size;					// Current size of the resource pool

	private Class template; 			// The resource class type

	private Recyclable store[];		// The resource pool
//	private boolean free[];			// Free resource flags

	private int inUse=0;				// Number of resources in use
	private int loop=0;				// Temp int

public Recycler(String typeName,int startSize,int maxSize) {
	super();

	this.startSize = startSize;
	this.maxSize = maxSize;
	this.size=startSize;

	store = new Recyclable[maxSize];
/*	free = new boolean[maxSize];

	// Set free flags false
	for(loop=0;loop<maxSize;loop++)
	{
		free[loop]=false;
	}*/

	// Create template class
	try
	{
		template=Class.forName(typeName);

		// Check template is recyclable
		if(!(template.newInstance() instanceof Recyclable))
		{
			System.out.println("Throw: Not recyclable");
			return;
		}
	}
	catch(Exception e)
	{
		System.out.println(e);
	}

	// Generate resource pool
	fill();
}
private void fill() {

	for(loop=0;loop<startSize;loop++)
	{
		try
		{
			store[loop]=(Recyclable)template.newInstance();
			store[loop].initialize();
			store[loop].setFree(true);
		}
		catch(Exception e)
		{
			System.out.println(e);
			return;
		}

		//free[loop]=true;
	}
}
public Object getClass(int index) {
	return store[index];
}
public Recyclable getInstance() {

	int index = getInstanceIndex();

	store[index].initialize();

	//System.out.println("Instance "+index+" of "+template.getName()+" taken");

	return store[index];
}
private synchronized int getInstanceIndex() {

	// Get next free instance
	for(loop=0;loop<size;loop++)
	{
		if(store[loop].isFree()==true)
		{
			store[loop].setFree(false);
			inUse++;
			return loop;
		}
	}

	// Increase avaiable resource size
	if(size<maxSize)
	{
		try
		{
			store[size]=(Recyclable)template.newInstance();
			store[size].initialize();
			store[size].setFree(false);
		}
		catch(Exception e)
		{
			System.out.println(e);
			return -1;
		}
		size++;

		inUse++;
		return size-1;
	}

	return -1;
}
public int getInUse() {
	return inUse;
}
public int getMaxSize() {
	return maxSize;
}
public int getSize() {
	return size;
}
public synchronized void release(Recycler instance) {

	for(loop=0;loop<size;loop++)
	{
		if(store[loop]==instance)
		{
			store[loop].recycle();
			store[loop].setFree(true);
			inUse--;
			return;
		}
	}

}
}
