package org.one.stone.soup.xapp;

import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.util.Hashtable;

import org.one.stone.soup.entity.KeyValuePair;
import org.one.stone.soup.stringhelper.StringArrayHelper;
import org.one.stone.soup.xapp.controller.xml.XmlApplication;
import org.one.stone.soup.xapp.javascript.JavascriptEngine;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlHelper;
import org.one.stone.soup.xml.helper.XmlPathSearch;

public class XappDB {

	private static JavascriptEngine defaultJSEngine = new JavascriptEngine();
	private static Hashtable interactiveValues = new Hashtable();

	public static XmlElement findElement(String query)
	{
		if(query==null)
		{
			try{ // to get stack trace
				throw new NullPointerException("Query is null");
			}
			catch(Exception e)
			{
				XappRootApplication.displayException(e);
			}
			return null;
		}
		else if(query.indexOf("xpath.application:")==0)
		{
			query = query.substring("xpath.application:".length());

			return XappRootApplication.getConfig().findElement( query );
		}
		else if(query.indexOf("xpath.model:")==0)
		{
			query = query.substring("xpath.model:".length());
			
			XmlPathSearch search = new XmlPathSearch(XmlApplication.getXmlData());
			return search.findElement( query );
		}

		return null;
	}
	
	public static XmlElement findElements(String query)
	{
		if(query==null)
		{
			try{ // to get stack trace
				throw new NullPointerException("Query is null");
			}
			catch(Exception e)
			{
				XappRootApplication.displayException(e);
			}
			return null;
		}
		else if(query.indexOf("xpath.application:")==0)
		{
			query = query.substring("xpath.application:".length());

			return XappRootApplication.getConfig().findElements( query );
		}
		else if(query.indexOf("xpath.model:")==0)
		{
			query = query.substring("xpath.model:".length());
			
			XmlPathSearch search = new XmlPathSearch(XmlApplication.getXmlData());
			return search.findElements( query );
		}

		return null;
	}
	
	public static String findValue(String query)
	{
		return findValue( query,defaultJSEngine );
	}
	
	public static String findValue(String query,JavascriptEngine jsEngine)
	{
		if(query==null)
		{
			try{ // to get stack trace
				throw new NullPointerException("Query is null");
			}
			catch(Exception e)
			{
				XappRootApplication.displayException(e);
			}
			return null;
		}
		else if(query.indexOf("xpath.application:")==0)
		{
			KeyValuePair request = getXPathQuery(query,"xpath.application:");
			
			if(request.value==null)
			{
				return XappRootApplication.getConfig().findElement( request.key ).getValue();
			}
			else
			{
				return XappRootApplication.getConfig().findElement( request.key ).getAttributeValueByName(request.value);				
			}
		}
		else if(query.indexOf("xpath.model:")==0)
		{
			KeyValuePair request = getXPathQuery(query,"xpath.model:");
			
			if( (XappRootApplication.getRoot() instanceof XmlApplication)==false )
			{
				return null;
			}
			else
			{
				XmlPathSearch search = new XmlPathSearch(XmlApplication.getXmlData());
				
				if(request.value==null)
				{
					return search.findElement( request.key ).getValue();
				}
				else
				{
					return search.findElement( request.key ).getAttributeValueByName(request.value);				
				}
			}
		}
		else if(query.indexOf("xapp:")==0)
		{
			query = query.substring(5);
			String[] args = StringArrayHelper.parseFields(query,'.');
			String formName = args[0];
			String formFieldName = args[1];
			return XappRootApplication.getAPI().getField(formName,formFieldName);
		}
		else if(query.indexOf("interactive:")==0)
		{
			query = query.substring(12);
			
			String value = (String)interactiveValues.get(query);
			value = XappRootApplication.askForValue(query,value);
			interactiveValues.put(query,value);
			
			return value;
		}
		else if(query.indexOf("javascript:")==0)
		{
			query = query.substring(11);
			return jsEngine.runJavascript(query);
		}

		return query;
	}
	
	public static Hashtable findList(String query)
	{
		return findList(query,defaultJSEngine);
	}
	
	public static Hashtable findList(String query,JavascriptEngine jsEngine)
	{
		if(query==null)
		{
			try{ // to get stack trace
				throw new NullPointerException("Query is null");
			}
			catch(Exception e)
			{
				XappRootApplication.displayException(e);
			}
			return null;
		}
		else if(query.indexOf("xpath.application:")==0)
		{
			KeyValuePair request = getXPathQuery(query,"xpath.application:");
			
			if(request.value==null)
			{
				return XmlHelper.getElementAsHashtable( XappRootApplication.getConfig().findElement( request.key ) );
			}
			else
			{
				return XmlHelper.getElementAsHashtable( XappRootApplication.getConfig().findElement( request.key ),request.value,request.value );				
			}
		}
		else if(query.indexOf("xpath.model:")==0)
		{
			KeyValuePair request = getXPathQuery(query,"xpath.model:");
			
			if( (XappRootApplication.getRoot() instanceof XmlApplication)==false )
			{
				return null;
			}
			else
			{
				XmlPathSearch search = new XmlPathSearch(XmlApplication.getXmlData());
				
				if(request.value==null)
				{
					return XmlHelper.getElementAsHashtable( search.findElements( request.key ) );
				}
				else
				{
					return XmlHelper.getElementAsHashtable( search.findElements( request.key ),request.value,request.value );				
				}
			}
		}
		else if(query.indexOf("xapp:")==0)
		{
			query = query.substring(5);
			
			String[] queries = StringArrayHelper.parseFields(query,';');
			
			Hashtable list = new Hashtable();
			for(int loop=0;loop<queries.length;loop++)
			{
				list.putAll( XmlHelper.getElementAsHashtable( XappRootApplication.getAPI().getFormData(queries[loop]) ) );
			}
			
			return list;
		}
		else if(query.indexOf("interactive:")==0)
		{
			char separator = query.charAt(12);
			query = query.substring(13);
			String[] items = StringArrayHelper.parseFields( query,separator );
			Hashtable data = new Hashtable();
			for(int loop=0;loop<items.length;loop++)
			{
				String key = items[loop];
				String value = (String)interactiveValues.get(key);
				value = XappRootApplication.askForValue("Enter value for "+key,value);
				if(value==null)
				{
					return null;
				}
				interactiveValues.put(key,value);
				data.put(key,value);
			}
			return data;
		}
		else if(query.indexOf("javascript:")==0)
		{
			char separator = query.charAt(12);
			query = query.substring(12);
			String result = jsEngine.runJavascript(query);
			
			String[] items = StringArrayHelper.parseFields(result,separator);
			Hashtable data = new Hashtable();
			for(int loop=0;loop<items.length;loop++)
			{
				String[] args = StringArrayHelper.parseFields( items[loop],'=' );
				data.put(args[0],args[1]);
			}
			return data;			
		}
		else if (query.indexOf("system:")==0)
		{
			query = query.substring(7);
			if(query.equals("fonts"))
			{
				 GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
				 Font[] fonts = ge.getAllFonts();
				 Hashtable data = new Hashtable();
				
				 String[] fontNames = new String[fonts.length];
				 for(int loop=0;loop<fonts.length;loop++)
				 {
					fontNames[loop] = fonts[loop].getFontName(); 
				 }				 
				 //fontNames = StringArrayHelper.sort(fontNames,StringArrayHelper.ASCENDING);
				 
				 for(int loop=0;loop<fonts.length;loop++)
				 {
					data.put(fontNames[loop],fontNames[loop]); 
				 }
				return data;
			}
			else if(query.equals("forms"))
			{
				String[] formNames = XappRootApplication.getFormDefinitionNames();
				
				 Hashtable data = new Hashtable();
				 for(int loop=0;loop<formNames.length;loop++)
				 {
					data.put(formNames[loop],formNames[loop]); 
				 }
				 return data;
			}
			return null;
		}
		else
		{
			char separator = query.charAt(0);
			String result = query.substring(1);
			
			String[] items = StringArrayHelper.parseFields(result,separator);
			Hashtable data = new Hashtable();
			for(int loop=0;loop<items.length;loop++)
			{
				String[] args = StringArrayHelper.parseFields( items[loop],'=' );
				data.put(args[0],args[1]);
			}
			return data;			
		}
	}	
	
	private static KeyValuePair getXPathQuery(String query,String type)
	{
		query = query.substring(type.length());
		
		int pos = query.lastIndexOf("@");
		String attributeName = null;
		if(pos>0)
		{
			attributeName = query.substring(pos+1);
			if(attributeName.indexOf(']')!=-1)
			{
				attributeName = null;
			}
			else
			{
				query = query.substring(0,pos); 
			}
		}
		
		return new KeyValuePair(query,attributeName);
	}
}
