package org.one.stone.soup.xapp;

import java.io.File;
import java.util.Hashtable;
import java.util.Vector;

import org.one.stone.soup.stringhelper.StringArrayHelper;
import org.one.stone.soup.xapp.components.XappComponent;
import org.one.stone.soup.xapp.containers.XappContainer;
import org.one.stone.soup.xapp.factories.XappComponentFactory;
import org.one.stone.soup.xapp.xpath.XappXPathDB;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlCloner;

public class XappRootApplication {

	private static Hashtable roots = new Hashtable();
	
	public static ThreadGroup getParentThreadGroup(Thread thread)
	{
		return thread.getThreadGroup().getParent();
	}
	
	public static void setRoot(XApplication newMain)
	{
		roots.put(getParentThreadGroup(Thread.currentThread()),newMain);
	}
	
	public static XApplication getRoot()
	{
		return (XApplication)roots.get( getParentThreadGroup(Thread.currentThread()) );
	}
	
	public static int getMainWidth()
	{
		return getRoot().getFrame().getWidth();
	}

	public static void setMainWidth(int width)
	{
		getRoot().getFrame().setSize(width,getRoot().getFrame().getHeight());
	}

	public static int getMainHeight()
	{
		return getRoot().getFrame().getHeight();
	}

	public static void setMainHeight(int height)
	{
		getRoot().getFrame().setSize(getRoot().getFrame().getWidth(),height);
	}

	public static XappComponentStore getComponentStore()
	{
		return getRoot().getComponentStore();
	}

	public static XappReceiver getReceiver()
	{
		return getRoot();
	}

	public static XappPluginStore getPluginStore()
	{
		return getRoot().getPluginStore();
	}
	
	public static XappPlugin loadPlugin(File pluginFile)
	{
		XmlElement xController = getRoot().getXAppConfig().getRoot().getElementByName("controller");
	
		XappPlugin plugin = getRoot().loadPlugin(pluginFile,xController);
		getRoot().getFrame().getXappMenuBar().refresh();
		
		return plugin;
	}

	public static void removePlugin(XappPlugin plugin)
	{
		getRoot().removePlugin(plugin,getRoot().getXAppConfig().getRoot().getElementByName("controller"));
	}
	
	public static XappXPathDB getConfig()
	{
		return getRoot().getXAppConfig();
	}

	public static XappSimpleAPI getAPI()
	{
		return new XappSimpleAPI(getRoot());
	}

	public static XappComponent getComponent(String componentName)
	{
		return getRoot().getComponentStore().getXappComponent(componentName);
	}

	public static void displayException(Exception e)
	{
		getRoot()._displayException(e);
	}
	
	public static void displayMessage(String message)
	{
		getRoot()._displayMessage(message);
	}
	
	public static boolean askForConfirmation(String message)
	{
		return getRoot().getFrame().askForConfirmation(message);
	}
	public static String askForValue(String query,String value)
	{
		return getRoot().getFrame().askForValue(query,value);
	}
	
	public static String askForSelection(String query,String title,String[] options,String value)
	{
		return getRoot().getFrame().askForSelection(query,title,options,value);
	}

	
	public static boolean autoSaveIsOn()
	{
		return getRoot().isAutoSaveOn();
	}

	public static void setAutosave(boolean state)
	{
		getRoot().setAutoSave(state);
	}

	public static String[] getFormDefinitionNames()
	{
		Vector names = new Vector();
		XmlElement result = getRoot().getXAppConfig().findElements("xapp/application/forms/form");
		for(int loop=0;loop<result.getElementCount();loop++)
		{
			names.addElement(result.getElementByIndex(loop).getAttributeValueByName("name"));
		}
		
		return StringArrayHelper.vectorToStringArray(names);
	}
	
	public static XmlElement getFormDefinition(String formName)
	{
		return XmlCloner.getClone( getRoot().getXAppConfig().findElement("xapp/application/form-definition/form[@name='"+formName+"']") );
	}

	public static XappComponentFactory getComponentFactory()
	{
		return getRoot().getComponentFactory();
	}
}
