package org.one.stone.soup.xapp;

import org.one.stone.soup.stringhelper.StringArrayHelper;
import org.one.stone.soup.xapp.components.form.XForm;
import org.one.stone.soup.xapp.components.spreadsheet.XappSpreadSheetAPI;
import org.one.stone.soup.xapp.components.spreadsheet.XappSpreadsheet;
import org.one.stone.soup.xapp.components.tree.XTree;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlPathSearch;

public class XappSimpleAPI {

	private XApplication application;
	
	public XappSimpleAPI(XApplication application)
	{
		this.application = application;
	}
	
	public String getApplicationTitle()
	{
		return application.getTitle();
	}
	
	public void displayException(String message)
	{
		XappRootApplication.displayException( new Exception(message) );
	}
	
	public void displayMessage(String message)
	{
		XappRootApplication.displayMessage( message );
	}
	
	public void doAction(String action)
	{
		application.processAction( "API",action );
	}
	
	public XmlElement getFormData(String formName)
	{
		return application.getFormData( formName );
	}
	
	public XappSpreadSheetAPI getSpreadsheet(String sheetName)
	{
		if( application.getComponentStore().getXappComponent(sheetName) instanceof XappSpreadsheet )
		{
			return new XappSpreadSheetAPI( ((XappSpreadsheet)application.getComponentStore().getXappComponent(sheetName)).getSpreadsheetModel() );
		}
		else
		{
			return null;
		}
	}

	public String getField(String formAndFieldName)
	{
		String[] args = StringArrayHelper.parseFields(formAndFieldName,'.');
		return getField(args[0],args[1]);
	}
	public String getField(String formName,String fieldName)
	{
		return getFormData(formName,fieldName);
	}
	
	public String getFormData(String formName,String fieldName)
	{
		if( application.getComponentStore().getXappComponent(formName) instanceof XappSpreadsheet )
		{
			return new XappSpreadSheetAPI( ((XappSpreadsheet)application.getComponentStore().getXappComponent(formName)).getSpreadsheetModel() ).getField(fieldName);
		}
		else if( application.getComponentStore().getXappComponent(formName) instanceof XTree )
		{
			XmlElement treeData = ((XTree)application.getComponentStore().getXappComponent(formName)).getData();
			return new XmlPathSearch(treeData).findElement(fieldName).getValue();
		}
				
		return application.getFormData( formName ).getElementByName(fieldName).getValue();
	}

	public void setFormData(String formName,XmlElement formData)
	{	
		((XForm)XappRootApplication.getComponent( formName )).setData(formData);
	}
	
	public void setField(String formAndFieldName,String value)
	{
		String[] args = StringArrayHelper.parseFields(formAndFieldName,'.');
		setField(args[0],args[1],value);
	}
	
	public void setField(String formName,String fieldName,String value)
	{
		setFormData(formName,fieldName,value);
	}

	public void incField(String formName,String fieldName)
	{
		int value = Integer.parseInt( getField(formName,fieldName) );
		value++;
		setField(formName,fieldName,""+value);
	}
	
	public void decField(String formName,String fieldName)
	{
		int value = Integer.parseInt( getField(formName,fieldName) );
		value--;
		setField(formName,fieldName,""+value);
	}
	
	public String call(String containerName,String method)
	{
		Object target = XappRootApplication.getComponent(containerName);
		if(target instanceof XappSpreadsheet)
		{
			target = ((XappSpreadsheet)target).getSpreadsheetModel();
		}
		try{
			return ""+target.getClass().getMethod(method,new Class[]{}).invoke(target,new Object[]{});
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return null;
	}
	
	public String call(String containerName,String method,String args)
	{
		Object target = XappRootApplication.getComponent(containerName);
		String[] data = StringArrayHelper.parseFields(args,' ');
		try{
			return ""+target.getClass().getMethod(method,new Class[]{String[].class}).invoke(target,new Object[]{data});
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return null;
	}	
	
	public void setFormData(String formName,String fieldName,String value)
	{
		if( application.getComponentStore().getXappComponent(formName) instanceof XappSpreadsheet )
		{
			new XappSpreadSheetAPI( ((XappSpreadsheet)application.getComponentStore().getXappComponent(formName)).getSpreadsheetModel() ).setField(fieldName,value);
			return;
		}
		else if( application.getComponentStore().getXappComponent(formName) instanceof XappPersister)
		{
			XmlElement data = new XmlElement("data");
			data.addChild(fieldName).addValue(value);

			XappPersister persister = (XappPersister)application.getComponentStore().getXappComponent(formName);
			persister.setData(data);
		}
		
		/*XmlElement data = new XmlElement("data");
		data.addChild(fieldName).addValue(value);
		XForm form = ((XForm)application.getComponent( formName ));
		if(form==null)
		{
			return;
		}
		form.setData(data);*/
	}		
}
