package org.one.stone.soup.xapp.controller.xml;

import org.one.stone.soup.clipboard.ClipboardHelper;
import org.one.stone.soup.xapp.XappRootApplication;
import org.one.stone.soup.xapp.containers.XappPopup;
import org.one.stone.soup.xml.XmlElement;
import org.one.stone.soup.xml.helper.XmlHelper;
import org.one.stone.soup.xml.stream.XmlLoader;

public class XmlPopupFactory {

	private XmlSchemaController schema;
	private XmlApplicationController controller;
	
	public XmlPopupFactory(XmlApplicationController controller,XmlSchemaController schema)
	{
		this.controller = controller;
		this.schema = schema;
	}
	
	
	public XappPopup createPopup(XmlElement root,XmlElement target)
	{
		XmlElement xPopup = new XmlElement("popup");
		xPopup.addAttribute("name","xmlPopup");

		XmlElement item  = xPopup.addChild("item");
		item.addAttribute("name","Cut");
		item.addAttribute("command","cut");

		item  = xPopup.addChild("item");
		item.addAttribute("name","Copy");
		item.addAttribute("command","copy");

		item  = xPopup.addChild("item");
		item.addAttribute("name","Paste");
		item.addAttribute("command","paste");
		XmlElement child = getClipboardXml();
		if(child==null || schema.canAddChild(root,target,child)==false)
		{
			item.addAttribute("enabled","false");
		}
		
		item  = xPopup.addChild("item");
		item.addAttribute("name","Remove");
		item.addAttribute("command","remove");
		if(  schema.canBeRemoved(root,target)==false )
		{
			item.addAttribute("enabled","false");
		}		
		
		XmlElement menu  = xPopup.addChild("menu");
		menu.addAttribute("name","Add Attribute");
		XmlElement xAttributes = schema.getPossibleAttributes(root,target);
		for(int loop=0;loop<xAttributes.getElementCount();loop++)
		{
			item = menu.addChild("item");
			item.addAttribute("name",xAttributes.getElementByIndex(loop).getAttributeValueByName("name"));
			item.addAttribute("command","addAttribute "+xAttributes.getElementByIndex(loop).getAttributeValueByName("name"));
		}
		if(xAttributes.getElementCount()==0)
		{
			menu.addAttribute("enabled","false");
		}

		menu  = xPopup.addChild("menu");
		menu.addAttribute("name","Remove Attribute");
		boolean hasRemovableAttributes = false;
		for(int loop=0;loop<target.getAttributeCount();loop++)
		{
			if(schema.canBeRemoved(root,target,target.getAttributeNameByIndex(loop))==false)
			{
				continue;
			}
			hasRemovableAttributes = true;
			item = menu.addChild("item");
			item.addAttribute("name",target.getAttributeNameByIndex(loop));
			item.addAttribute("command","removeAttribute "+target.getAttributeNameByIndex(loop));
		}
		if(hasRemovableAttributes==false)
		{
			menu.addAttribute("enabled","false");
		}
		
		menu = xPopup.addChild("menu");
		menu.addAttribute("name","Add Child");
		XmlElement xNodes = schema.getPossibleChildren(root,target);
		for(int loop=0;loop<xNodes.getElementCount();loop++)
		{
			item = menu.addChild("item");
			item.addAttribute("name",xNodes.getElementByIndex(loop).getAttributeValueByName("name"));
			item.addAttribute("command","addNode "+xNodes.getElementByIndex(loop).getAttributeValueByName("name"));
		}
		if(xNodes.getElementCount()==0)
		{
			menu.addAttribute("enabled","false");
		}
		
		if( schema.isParentOrdered(root,target) )
		{
			item = xPopup.addChild("item");
			item .addAttribute("name","Move Up");
			item .addAttribute("command","moveUp");
			if(schema.canMoveUp(root,target)==false)
			{
				item .addAttribute("enabled","false");
			}
			
			item = xPopup.addChild("item");
			item .addAttribute("name","Move Down");
			item .addAttribute("command","moveDown");	
			if(schema.canMoveDown(root,target)==false)
			{
				item .addAttribute("enabled","false");
			}
		}
		
		return XappRootApplication.getComponentFactory().buildXappPopup(xPopup);
	}
	
	public void processAction(XmlElement dataRoot,XmlElement dataSource,XmlElement action)
	{		
		if(action.getValue().indexOf("addAttribute")==0)
		{
			String attributeName = action.getValue().substring(13);
			//dataSource.addAttribute(attributeName,"");
			schema.addAttribute(dataRoot,dataSource,attributeName);
		}
		else if(action.getValue().indexOf("removeAttribute")==0)
		{
			String attributeName = action.getValue().substring(16);
			dataSource.removeAttribute(attributeName);	
		}
		else if(action.getValue().indexOf("addNode")==0)
		{
			String nodeName = action.getValue().substring(8);
			dataSource = schema.addChild(dataRoot,dataSource,nodeName);			
		}
		else if(action.getValue().equals("moveUp"))
		{
			schema.moveUp( dataRoot,dataSource );
		}
		else if(action.getValue().equals("moveDown"))
		{
			schema.moveDown( dataRoot,dataSource );
		}
		else if(action.getValue().equals("remove"))
		{
			XmlElement parent = XmlHelper.getParent(dataRoot,dataSource);
			schema.remove( dataRoot,dataSource );
			dataSource = parent;
		}
		else if(action.getValue().equals("cut"))
		{
			XmlElement parent = XmlHelper.getParent(dataRoot,dataSource);
			ClipboardHelper.toClipboard( dataSource.toXml() );
			schema.remove( dataRoot,dataSource );
			dataSource = parent;
		}
		else if(action.getValue().equals("copy"))
		{
			ClipboardHelper.toClipboard( dataSource.toXml() );
		}
		else if(action.getValue().equals("paste"))
		{
			XmlElement child = getClipboardXml();
			if(child==null)
			{
				return;
			}
			dataSource = schema.addChild( dataRoot,dataSource,child );
		}

		controller.setContext(dataSource);
		XmlApplication.openForm(dataRoot,dataSource);		
	}
	
	private XmlElement getClipboardXml()
	{
		try{
			XmlElement child = XmlLoader.getStandardParser().parseElement( ClipboardHelper.fromClipboard() );
			return child;
		}
		catch(Exception e)
		{
			return null;
		}
	}
}
